# /* **************************************************************************
#  *                                                                          *
#  *     (C) Copyright Paul Mensonides 2002.
#  *     Distributed under the Boost Software License, Version 1.0. (See
#  *     accompanying file LICENSE_1_0.txt or copy at
#  *     http://www.boost.org/LICENSE_1_0.txt)
#  *                                                                          *
#  ************************************************************************** */
#
# /* Revised by Edward Diener (2020) */
#
# /* See http://www.boost.org for most recent version. */
#
# ifndef BOOST_PREPROCESSOR_FACILITIES_INTERCEPT_1024_HPP
# define BOOST_PREPROCESSOR_FACILITIES_INTERCEPT_1024_HPP
#
# define BOOST_PP_INTERCEPT_513
# define BOOST_PP_INTERCEPT_514
# define BOOST_PP_INTERCEPT_515
# define BOOST_PP_INTERCEPT_516
# define BOOST_PP_INTERCEPT_517
# define BOOST_PP_INTERCEPT_518
# define BOOST_PP_INTERCEPT_519
# define BOOST_PP_INTERCEPT_520
# define BOOST_PP_INTERCEPT_521
# define BOOST_PP_INTERCEPT_522
# define BOOST_PP_INTERCEPT_523
# define BOOST_PP_INTERCEPT_524
# define BOOST_PP_INTERCEPT_525
# define BOOST_PP_INTERCEPT_526
# define BOOST_PP_INTERCEPT_527
# define BOOST_PP_INTERCEPT_528
# define BOOST_PP_INTERCEPT_529
# define BOOST_PP_INTERCEPT_530
# define BOOST_PP_INTERCEPT_531
# define BOOST_PP_INTERCEPT_532
# define BOOST_PP_INTERCEPT_533
# define BOOST_PP_INTERCEPT_534
# define BOOST_PP_INTERCEPT_535
# define BOOST_PP_INTERCEPT_536
# define BOOST_PP_INTERCEPT_537
# define BOOST_PP_INTERCEPT_538
# define BOOST_PP_INTERCEPT_539
# define BOOST_PP_INTERCEPT_540
# define BOOST_PP_INTERCEPT_541
# define BOOST_PP_INTERCEPT_542
# define BOOST_PP_INTERCEPT_543
# define BOOST_PP_INTERCEPT_544
# define BOOST_PP_INTERCEPT_545
# define BOOST_PP_INTERCEPT_546
# define BOOST_PP_INTERCEPT_547
# define BOOST_PP_INTERCEPT_548
# define BOOST_PP_INTERCEPT_549
# define BOOST_PP_INTERCEPT_550
# define BOOST_PP_INTERCEPT_551
# define BOOST_PP_INTERCEPT_552
# define BOOST_PP_INTERCEPT_553
# define BOOST_PP_INTERCEPT_554
# define BOOST_PP_INTERCEPT_555
# define BOOST_PP_INTERCEPT_556
# define BOOST_PP_INTERCEPT_557
# define BOOST_PP_INTERCEPT_558
# define BOOST_PP_INTERCEPT_559
# define BOOST_PP_INTERCEPT_560
# define BOOST_PP_INTERCEPT_561
# define BOOST_PP_INTERCEPT_562
# define BOOST_PP_INTERCEPT_563
# define BOOST_PP_INTERCEPT_564
# define BOOST_PP_INTERCEPT_565
# define BOOST_PP_INTERCEPT_566
# define BOOST_PP_INTERCEPT_567
# define BOOST_PP_INTERCEPT_568
# define BOOST_PP_INTERCEPT_569
# define BOOST_PP_INTERCEPT_570
# define BOOST_PP_INTERCEPT_571
# define BOOST_PP_INTERCEPT_572
# define BOOST_PP_INTERCEPT_573
# define BOOST_PP_INTERCEPT_574
# define BOOST_PP_INTERCEPT_575
# define BOOST_PP_INTERCEPT_576
# define BOOST_PP_INTERCEPT_577
# define BOOST_PP_INTERCEPT_578
# define BOOST_PP_INTERCEPT_579
# define BOOST_PP_INTERCEPT_580
# define BOOST_PP_INTERCEPT_581
# define BOOST_PP_INTERCEPT_582
# define BOOST_PP_INTERCEPT_583
# define BOOST_PP_INTERCEPT_584
# define BOOST_PP_INTERCEPT_585
# define BOOST_PP_INTERCEPT_586
# define BOOST_PP_INTERCEPT_587
# define BOOST_PP_INTERCEPT_588
# define BOOST_PP_INTERCEPT_589
# define BOOST_PP_INTERCEPT_590
# define BOOST_PP_INTERCEPT_591
# define BOOST_PP_INTERCEPT_592
# define BOOST_PP_INTERCEPT_593
# define BOOST_PP_INTERCEPT_594
# define BOOST_PP_INTERCEPT_595
# define BOOST_PP_INTERCEPT_596
# define BOOST_PP_INTERCEPT_597
# define BOOST_PP_INTERCEPT_598
# define BOOST_PP_INTERCEPT_599
# define BOOST_PP_INTERCEPT_600
# define BOOST_PP_INTERCEPT_601
# define BOOST_PP_INTERCEPT_602
# define BOOST_PP_INTERCEPT_603
# define BOOST_PP_INTERCEPT_604
# define BOOST_PP_INTERCEPT_605
# define BOOST_PP_INTERCEPT_606
# define BOOST_PP_INTERCEPT_607
# define BOOST_PP_INTERCEPT_608
# define BOOST_PP_INTERCEPT_609
# define BOOST_PP_INTERCEPT_610
# define BOOST_PP_INTERCEPT_611
# define BOOST_PP_INTERCEPT_612
# define BOOST_PP_INTERCEPT_613
# define BOOST_PP_INTERCEPT_614
# define BOOST_PP_INTERCEPT_615
# define BOOST_PP_INTERCEPT_616
# define BOOST_PP_INTERCEPT_617
# define BOOST_PP_INTERCEPT_618
# define BOOST_PP_INTERCEPT_619
# define BOOST_PP_INTERCEPT_620
# define BOOST_PP_INTERCEPT_621
# define BOOST_PP_INTERCEPT_622
# define BOOST_PP_INTERCEPT_623
# define BOOST_PP_INTERCEPT_624
# define BOOST_PP_INTERCEPT_625
# define BOOST_PP_INTERCEPT_626
# define BOOST_PP_INTERCEPT_627
# define BOOST_PP_INTERCEPT_628
# define BOOST_PP_INTERCEPT_629
# define BOOST_PP_INTERCEPT_630
# define BOOST_PP_INTERCEPT_631
# define BOOST_PP_INTERCEPT_632
# define BOOST_PP_INTERCEPT_633
# define BOOST_PP_INTERCEPT_634
# define BOOST_PP_INTERCEPT_635
# define BOOST_PP_INTERCEPT_636
# define BOOST_PP_INTERCEPT_637
# define BOOST_PP_INTERCEPT_638
# define BOOST_PP_INTERCEPT_639
# define BOOST_PP_INTERCEPT_640
# define BOOST_PP_INTERCEPT_641
# define BOOST_PP_INTERCEPT_642
# define BOOST_PP_INTERCEPT_643
# define BOOST_PP_INTERCEPT_644
# define BOOST_PP_INTERCEPT_645
# define BOOST_PP_INTERCEPT_646
# define BOOST_PP_INTERCEPT_647
# define BOOST_PP_INTERCEPT_648
# define BOOST_PP_INTERCEPT_649
# define BOOST_PP_INTERCEPT_650
# define BOOST_PP_INTERCEPT_651
# define BOOST_PP_INTERCEPT_652
# define BOOST_PP_INTERCEPT_653
# define BOOST_PP_INTERCEPT_654
# define BOOST_PP_INTERCEPT_655
# define BOOST_PP_INTERCEPT_656
# define BOOST_PP_INTERCEPT_657
# define BOOST_PP_INTERCEPT_658
# define BOOST_PP_INTERCEPT_659
# define BOOST_PP_INTERCEPT_660
# define BOOST_PP_INTERCEPT_661
# define BOOST_PP_INTERCEPT_662
# define BOOST_PP_INTERCEPT_663
# define BOOST_PP_INTERCEPT_664
# define BOOST_PP_INTERCEPT_665
# define BOOST_PP_INTERCEPT_666
# define BOOST_PP_INTERCEPT_667
# define BOOST_PP_INTERCEPT_668
# define BOOST_PP_INTERCEPT_669
# define BOOST_PP_INTERCEPT_670
# define BOOST_PP_INTERCEPT_671
# define BOOST_PP_INTERCEPT_672
# define BOOST_PP_INTERCEPT_673
# define BOOST_PP_INTERCEPT_674
# define BOOST_PP_INTERCEPT_675
# define BOOST_PP_INTERCEPT_676
# define BOOST_PP_INTERCEPT_677
# define BOOST_PP_INTERCEPT_678
# define BOOST_PP_INTERCEPT_679
# define BOOST_PP_INTERCEPT_680
# define BOOST_PP_INTERCEPT_681
# define BOOST_PP_INTERCEPT_682
# define BOOST_PP_INTERCEPT_683
# define BOOST_PP_INTERCEPT_684
# define BOOST_PP_INTERCEPT_685
# define BOOST_PP_INTERCEPT_686
# define BOOST_PP_INTERCEPT_687
# define BOOST_PP_INTERCEPT_688
# define BOOST_PP_INTERCEPT_689
# define BOOST_PP_INTERCEPT_690
# define BOOST_PP_INTERCEPT_691
# define BOOST_PP_INTERCEPT_692
# define BOOST_PP_INTERCEPT_693
# define BOOST_PP_INTERCEPT_694
# define BOOST_PP_INTERCEPT_695
# define BOOST_PP_INTERCEPT_696
# define BOOST_PP_INTERCEPT_697
# define BOOST_PP_INTERCEPT_698
# define BOOST_PP_INTERCEPT_699
# define BOOST_PP_INTERCEPT_700
# define BOOST_PP_INTERCEPT_701
# define BOOST_PP_INTERCEPT_702
# define BOOST_PP_INTERCEPT_703
# define BOOST_PP_INTERCEPT_704
# define BOOST_PP_INTERCEPT_705
# define BOOST_PP_INTERCEPT_706
# define BOOST_PP_INTERCEPT_707
# define BOOST_PP_INTERCEPT_708
# define BOOST_PP_INTERCEPT_709
# define BOOST_PP_INTERCEPT_710
# define BOOST_PP_INTERCEPT_711
# define BOOST_PP_INTERCEPT_712
# define BOOST_PP_INTERCEPT_713
# define BOOST_PP_INTERCEPT_714
# define BOOST_PP_INTERCEPT_715
# define BOOST_PP_INTERCEPT_716
# define BOOST_PP_INTERCEPT_717
# define BOOST_PP_INTERCEPT_718
# define BOOST_PP_INTERCEPT_719
# define BOOST_PP_INTERCEPT_720
# define BOOST_PP_INTERCEPT_721
# define BOOST_PP_INTERCEPT_722
# define BOOST_PP_INTERCEPT_723
# define BOOST_PP_INTERCEPT_724
# define BOOST_PP_INTERCEPT_725
# define BOOST_PP_INTERCEPT_726
# define BOOST_PP_INTERCEPT_727
# define BOOST_PP_INTERCEPT_728
# define BOOST_PP_INTERCEPT_729
# define BOOST_PP_INTERCEPT_730
# define BOOST_PP_INTERCEPT_731
# define BOOST_PP_INTERCEPT_732
# define BOOST_PP_INTERCEPT_733
# define BOOST_PP_INTERCEPT_734
# define BOOST_PP_INTERCEPT_735
# define BOOST_PP_INTERCEPT_736
# define BOOST_PP_INTERCEPT_737
# define BOOST_PP_INTERCEPT_738
# define BOOST_PP_INTERCEPT_739
# define BOOST_PP_INTERCEPT_740
# define BOOST_PP_INTERCEPT_741
# define BOOST_PP_INTERCEPT_742
# define BOOST_PP_INTERCEPT_743
# define BOOST_PP_INTERCEPT_744
# define BOOST_PP_INTERCEPT_745
# define BOOST_PP_INTERCEPT_746
# define BOOST_PP_INTERCEPT_747
# define BOOST_PP_INTERCEPT_748
# define BOOST_PP_INTERCEPT_749
# define BOOST_PP_INTERCEPT_750
# define BOOST_PP_INTERCEPT_751
# define BOOST_PP_INTERCEPT_752
# define BOOST_PP_INTERCEPT_753
# define BOOST_PP_INTERCEPT_754
# define BOOST_PP_INTERCEPT_755
# define BOOST_PP_INTERCEPT_756
# define BOOST_PP_INTERCEPT_757
# define BOOST_PP_INTERCEPT_758
# define BOOST_PP_INTERCEPT_759
# define BOOST_PP_INTERCEPT_760
# define BOOST_PP_INTERCEPT_761
# define BOOST_PP_INTERCEPT_762
# define BOOST_PP_INTERCEPT_763
# define BOOST_PP_INTERCEPT_764
# define BOOST_PP_INTERCEPT_765
# define BOOST_PP_INTERCEPT_766
# define BOOST_PP_INTERCEPT_767
# define BOOST_PP_INTERCEPT_768
# define BOOST_PP_INTERCEPT_769
# define BOOST_PP_INTERCEPT_770
# define BOOST_PP_INTERCEPT_771
# define BOOST_PP_INTERCEPT_772
# define BOOST_PP_INTERCEPT_773
# define BOOST_PP_INTERCEPT_774
# define BOOST_PP_INTERCEPT_775
# define BOOST_PP_INTERCEPT_776
# define BOOST_PP_INTERCEPT_777
# define BOOST_PP_INTERCEPT_778
# define BOOST_PP_INTERCEPT_779
# define BOOST_PP_INTERCEPT_780
# define BOOST_PP_INTERCEPT_781
# define BOOST_PP_INTERCEPT_782
# define BOOST_PP_INTERCEPT_783
# define BOOST_PP_INTERCEPT_784
# define BOOST_PP_INTERCEPT_785
# define BOOST_PP_INTERCEPT_786
# define BOOST_PP_INTERCEPT_787
# define BOOST_PP_INTERCEPT_788
# define BOOST_PP_INTERCEPT_789
# define BOOST_PP_INTERCEPT_790
# define BOOST_PP_INTERCEPT_791
# define BOOST_PP_INTERCEPT_792
# define BOOST_PP_INTERCEPT_793
# define BOOST_PP_INTERCEPT_794
# define BOOST_PP_INTERCEPT_795
# define BOOST_PP_INTERCEPT_796
# define BOOST_PP_INTERCEPT_797
# define BOOST_PP_INTERCEPT_798
# define BOOST_PP_INTERCEPT_799
# define BOOST_PP_INTERCEPT_800
# define BOOST_PP_INTERCEPT_801
# define BOOST_PP_INTERCEPT_802
# define BOOST_PP_INTERCEPT_803
# define BOOST_PP_INTERCEPT_804
# define BOOST_PP_INTERCEPT_805
# define BOOST_PP_INTERCEPT_806
# define BOOST_PP_INTERCEPT_807
# define BOOST_PP_INTERCEPT_808
# define BOOST_PP_INTERCEPT_809
# define BOOST_PP_INTERCEPT_810
# define BOOST_PP_INTERCEPT_811
# define BOOST_PP_INTERCEPT_812
# define BOOST_PP_INTERCEPT_813
# define BOOST_PP_INTERCEPT_814
# define BOOST_PP_INTERCEPT_815
# define BOOST_PP_INTERCEPT_816
# define BOOST_PP_INTERCEPT_817
# define BOOST_PP_INTERCEPT_818
# define BOOST_PP_INTERCEPT_819
# define BOOST_PP_INTERCEPT_820
# define BOOST_PP_INTERCEPT_821
# define BOOST_PP_INTERCEPT_822
# define BOOST_PP_INTERCEPT_823
# define BOOST_PP_INTERCEPT_824
# define BOOST_PP_INTERCEPT_825
# define BOOST_PP_INTERCEPT_826
# define BOOST_PP_INTERCEPT_827
# define BOOST_PP_INTERCEPT_828
# define BOOST_PP_INTERCEPT_829
# define BOOST_PP_INTERCEPT_830
# define BOOST_PP_INTERCEPT_831
# define BOOST_PP_INTERCEPT_832
# define BOOST_PP_INTERCEPT_833
# define BOOST_PP_INTERCEPT_834
# define BOOST_PP_INTERCEPT_835
# define BOOST_PP_INTERCEPT_836
# define BOOST_PP_INTERCEPT_837
# define BOOST_PP_INTERCEPT_838
# define BOOST_PP_INTERCEPT_839
# define BOOST_PP_INTERCEPT_840
# define BOOST_PP_INTERCEPT_841
# define BOOST_PP_INTERCEPT_842
# define BOOST_PP_INTERCEPT_843
# define BOOST_PP_INTERCEPT_844
# define BOOST_PP_INTERCEPT_845
# define BOOST_PP_INTERCEPT_846
# define BOOST_PP_INTERCEPT_847
# define BOOST_PP_INTERCEPT_848
# define BOOST_PP_INTERCEPT_849
# define BOOST_PP_INTERCEPT_850
# define BOOST_PP_INTERCEPT_851
# define BOOST_PP_INTERCEPT_852
# define BOOST_PP_INTERCEPT_853
# define BOOST_PP_INTERCEPT_854
# define BOOST_PP_INTERCEPT_855
# define BOOST_PP_INTERCEPT_856
# define BOOST_PP_INTERCEPT_857
# define BOOST_PP_INTERCEPT_858
# define BOOST_PP_INTERCEPT_859
# define BOOST_PP_INTERCEPT_860
# define BOOST_PP_INTERCEPT_861
# define BOOST_PP_INTERCEPT_862
# define BOOST_PP_INTERCEPT_863
# define BOOST_PP_INTERCEPT_864
# define BOOST_PP_INTERCEPT_865
# define BOOST_PP_INTERCEPT_866
# define BOOST_PP_INTERCEPT_867
# define BOOST_PP_INTERCEPT_868
# define BOOST_PP_INTERCEPT_869
# define BOOST_PP_INTERCEPT_870
# define BOOST_PP_INTERCEPT_871
# define BOOST_PP_INTERCEPT_872
# define BOOST_PP_INTERCEPT_873
# define BOOST_PP_INTERCEPT_874
# define BOOST_PP_INTERCEPT_875
# define BOOST_PP_INTERCEPT_876
# define BOOST_PP_INTERCEPT_877
# define BOOST_PP_INTERCEPT_878
# define BOOST_PP_INTERCEPT_879
# define BOOST_PP_INTERCEPT_880
# define BOOST_PP_INTERCEPT_881
# define BOOST_PP_INTERCEPT_882
# define BOOST_PP_INTERCEPT_883
# define BOOST_PP_INTERCEPT_884
# define BOOST_PP_INTERCEPT_885
# define BOOST_PP_INTERCEPT_886
# define BOOST_PP_INTERCEPT_887
# define BOOST_PP_INTERCEPT_888
# define BOOST_PP_INTERCEPT_889
# define BOOST_PP_INTERCEPT_890
# define BOOST_PP_INTERCEPT_891
# define BOOST_PP_INTERCEPT_892
# define BOOST_PP_INTERCEPT_893
# define BOOST_PP_INTERCEPT_894
# define BOOST_PP_INTERCEPT_895
# define BOOST_PP_INTERCEPT_896
# define BOOST_PP_INTERCEPT_897
# define BOOST_PP_INTERCEPT_898
# define BOOST_PP_INTERCEPT_899
# define BOOST_PP_INTERCEPT_900
# define BOOST_PP_INTERCEPT_901
# define BOOST_PP_INTERCEPT_902
# define BOOST_PP_INTERCEPT_903
# define BOOST_PP_INTERCEPT_904
# define BOOST_PP_INTERCEPT_905
# define BOOST_PP_INTERCEPT_906
# define BOOST_PP_INTERCEPT_907
# define BOOST_PP_INTERCEPT_908
# define BOOST_PP_INTERCEPT_909
# define BOOST_PP_INTERCEPT_910
# define BOOST_PP_INTERCEPT_911
# define BOOST_PP_INTERCEPT_912
# define BOOST_PP_INTERCEPT_913
# define BOOST_PP_INTERCEPT_914
# define BOOST_PP_INTERCEPT_915
# define BOOST_PP_INTERCEPT_916
# define BOOST_PP_INTERCEPT_917
# define BOOST_PP_INTERCEPT_918
# define BOOST_PP_INTERCEPT_919
# define BOOST_PP_INTERCEPT_920
# define BOOST_PP_INTERCEPT_921
# define BOOST_PP_INTERCEPT_922
# define BOOST_PP_INTERCEPT_923
# define BOOST_PP_INTERCEPT_924
# define BOOST_PP_INTERCEPT_925
# define BOOST_PP_INTERCEPT_926
# define BOOST_PP_INTERCEPT_927
# define BOOST_PP_INTERCEPT_928
# define BOOST_PP_INTERCEPT_929
# define BOOST_PP_INTERCEPT_930
# define BOOST_PP_INTERCEPT_931
# define BOOST_PP_INTERCEPT_932
# define BOOST_PP_INTERCEPT_933
# define BOOST_PP_INTERCEPT_934
# define BOOST_PP_INTERCEPT_935
# define BOOST_PP_INTERCEPT_936
# define BOOST_PP_INTERCEPT_937
# define BOOST_PP_INTERCEPT_938
# define BOOST_PP_INTERCEPT_939
# define BOOST_PP_INTERCEPT_940
# define BOOST_PP_INTERCEPT_941
# define BOOST_PP_INTERCEPT_942
# define BOOST_PP_INTERCEPT_943
# define BOOST_PP_INTERCEPT_944
# define BOOST_PP_INTERCEPT_945
# define BOOST_PP_INTERCEPT_946
# define BOOST_PP_INTERCEPT_947
# define BOOST_PP_INTERCEPT_948
# define BOOST_PP_INTERCEPT_949
# define BOOST_PP_INTERCEPT_950
# define BOOST_PP_INTERCEPT_951
# define BOOST_PP_INTERCEPT_952
# define BOOST_PP_INTERCEPT_953
# define BOOST_PP_INTERCEPT_954
# define BOOST_PP_INTERCEPT_955
# define BOOST_PP_INTERCEPT_956
# define BOOST_PP_INTERCEPT_957
# define BOOST_PP_INTERCEPT_958
# define BOOST_PP_INTERCEPT_959
# define BOOST_PP_INTERCEPT_960
# define BOOST_PP_INTERCEPT_961
# define BOOST_PP_INTERCEPT_962
# define BOOST_PP_INTERCEPT_963
# define BOOST_PP_INTERCEPT_964
# define BOOST_PP_INTERCEPT_965
# define BOOST_PP_INTERCEPT_966
# define BOOST_PP_INTERCEPT_967
# define BOOST_PP_INTERCEPT_968
# define BOOST_PP_INTERCEPT_969
# define BOOST_PP_INTERCEPT_970
# define BOOST_PP_INTERCEPT_971
# define BOOST_PP_INTERCEPT_972
# define BOOST_PP_INTERCEPT_973
# define BOOST_PP_INTERCEPT_974
# define BOOST_PP_INTERCEPT_975
# define BOOST_PP_INTERCEPT_976
# define BOOST_PP_INTERCEPT_977
# define BOOST_PP_INTERCEPT_978
# define BOOST_PP_INTERCEPT_979
# define BOOST_PP_INTERCEPT_980
# define BOOST_PP_INTERCEPT_981
# define BOOST_PP_INTERCEPT_982
# define BOOST_PP_INTERCEPT_983
# define BOOST_PP_INTERCEPT_984
# define BOOST_PP_INTERCEPT_985
# define BOOST_PP_INTERCEPT_986
# define BOOST_PP_INTERCEPT_987
# define BOOST_PP_INTERCEPT_988
# define BOOST_PP_INTERCEPT_989
# define BOOST_PP_INTERCEPT_990
# define BOOST_PP_INTERCEPT_991
# define BOOST_PP_INTERCEPT_992
# define BOOST_PP_INTERCEPT_993
# define BOOST_PP_INTERCEPT_994
# define BOOST_PP_INTERCEPT_995
# define BOOST_PP_INTERCEPT_996
# define BOOST_PP_INTERCEPT_997
# define BOOST_PP_INTERCEPT_998
# define BOOST_PP_INTERCEPT_999
# define BOOST_PP_INTERCEPT_1000
# define BOOST_PP_INTERCEPT_1001
# define BOOST_PP_INTERCEPT_1002
# define BOOST_PP_INTERCEPT_1003
# define BOOST_PP_INTERCEPT_1004
# define BOOST_PP_INTERCEPT_1005
# define BOOST_PP_INTERCEPT_1006
# define BOOST_PP_INTERCEPT_1007
# define BOOST_PP_INTERCEPT_1008
# define BOOST_PP_INTERCEPT_1009
# define BOOST_PP_INTERCEPT_1010
# define BOOST_PP_INTERCEPT_1011
# define BOOST_PP_INTERCEPT_1012
# define BOOST_PP_INTERCEPT_1013
# define BOOST_PP_INTERCEPT_1014
# define BOOST_PP_INTERCEPT_1015
# define BOOST_PP_INTERCEPT_1016
# define BOOST_PP_INTERCEPT_1017
# define BOOST_PP_INTERCEPT_1018
# define BOOST_PP_INTERCEPT_1019
# define BOOST_PP_INTERCEPT_1020
# define BOOST_PP_INTERCEPT_1021
# define BOOST_PP_INTERCEPT_1022
# define BOOST_PP_INTERCEPT_1023
# define BOOST_PP_INTERCEPT_1024
#
# endif
